/* Copyright (c) 2013 Alexander Sherstnev
 * Copyright (c) 2011 Peter Troshin
 *  
 *  JAva Bioinformatics Analysis Web Services (JABAWS) @version: 2.0     
 * 
 *  This library is free software; you can redistribute it and/or modify it under the terms of the
 *  Apache License version 2 as published by the Apache Software Foundation
 * 
 *  This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Apache 
 *  License for more details.
 * 
 *  A copy of the license is in apache_license.txt. It is also available here:
 * @see: http://www.apache.org/licenses/LICENSE-2.0.txt
 * 
 * Any republication or derived work distributed in source code form
 * must include this copyright and license notice.
 */

package compbio.stat.servlet;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.lang.management.ManagementFactory;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.Collections;

import javax.management.AttributeNotFoundException;
import javax.management.InstanceNotFoundException;
import javax.management.MBeanException;
import javax.management.MBeanServer;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.management.Query;
import javax.management.ReflectionException;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;

import compbio.ws.client.Services;
import compbio.ws.client.WSTester;

/**
 * Use cases:
 * <dl>
 * <li>Test web services and display results on the web page</li>
 * </dl>
 * 
 * @author pvtroshin
 * 
 */

public class ServiceStatus extends HttpServlet {

	private final static Logger log = Logger.getLogger(ServiceStatus.class);

	private List<String> getEndPoints() throws MalformedObjectNameException, NullPointerException, UnknownHostException,
			AttributeNotFoundException, InstanceNotFoundException, MBeanException, ReflectionException {
		MBeanServer mbs = ManagementFactory.getPlatformMBeanServer();
		Set<ObjectName> objs = mbs.queryNames(new ObjectName("*:type=Connector,*"),
				Query.match(Query.attr("protocol"), Query.value("HTTP/1.1")));
		List<String> endPoints = new ArrayList<String>();
		for (Iterator<ObjectName> i = objs.iterator(); i.hasNext();) {
			ObjectName obj = i.next();
			String scheme = mbs.getAttribute(obj, "scheme").toString();
			String port = obj.getKeyProperty("port");
			String hostname = InetAddress.getLocalHost().getHostName();
			endPoints.add(scheme + "://" + hostname + ":" + port);
		}
		return endPoints;
	}

	@Override
	protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
		/**
		 * // PROBLEM: the code tries to test not WS endpoints on an internal
		 * Tomcat, but on an external // endpoints with wrong info on the
		 * proxing StringBuffer jabawspath = req.getRequestURL(); jabawspath =
		 * jabawspath.delete(jabawspath.lastIndexOf("/"), jabawspath.length());
		 * String serverPath = jabawspath.toString();
		 */
		List<String> eps = new ArrayList<String>();
		try {
			eps = getEndPoints();
		} catch (MalformedObjectNameException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (AttributeNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InstanceNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NullPointerException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (MBeanException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ReflectionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		if (1 != eps.size()) {
			log.info(eps.size() + "EndPoints found");
			//System.out.println("      " + eps.size() + " EndPoints found");
			for (String endpoint : eps) {
				log.info(eps.size() + "EndPoint is " + endpoint);
				//System.out.println("   EndPoint is " + endpoint);
			}
		}

		String serverPath = new String();
		for (String endpoint : eps) {
			serverPath = endpoint + req.getContextPath();
		}
		//System.out.println("Testing services at " + serverPath);

		List<ServiceTestResult> testResults = new ArrayList<ServiceTestResult>();
		long startTime = System.nanoTime();
		for (Services service : Services.values()) {
			StringWriter testres = new StringWriter();
			PrintWriter writer = new PrintWriter(testres, true);
			WSTester tester = new WSTester(serverPath, writer);
			ServiceTestResult result = new ServiceTestResult(service);
			try {
				result.failed = tester.checkService(service);
			} catch (Exception e) {
				log.info(e, e.getCause());
				String mess = "Fails to connect to the web service: " + service + ". Reason: ";
				writer.println(mess + e.getLocalizedMessage() + "\nDetails: ");
				e.printStackTrace(writer);
			} finally {
				writer.close();
			}
			result.details = testres.toString();
			testResults.add(result);
		}
		req.setAttribute("results", testResults);
		long endTime = System.nanoTime();
		req.setAttribute("timeexec", (endTime - startTime) / 1000000);
		RequestDispatcher rd = req.getRequestDispatcher("statpages/ServicesStatus.jsp");
		rd.forward(req, resp);
	}

}
